@@ -237,7 +237,7 @@ enum ElementState {
 	Close,
 	/// Delimiter, Alphabet and whether we just read a CR, because of the mess
 	/// which is CRLF -> LF normalization.
-	AttributeValue(u8, &'static [ByteRange], bool),
+	AttributeValue(u8, bool),
 	/// Encountered ?
 	MaybeXMLDeclEnd,
 	/// Encountered /
@@ -284,16 +284,16 @@ enum ContentState {
 
 #[derive(Debug, Clone, Copy, PartialEq)]
 enum RefReturnState {
-	AttributeValue(ElementKind, u8, &'static [ByteRange]),
+	AttributeValue(ElementKind, u8),
 	Text,
 }
 
 impl RefReturnState {
 	fn to_state(self) -> State {
 		match self {
-			Self::AttributeValue(kind, delim, selector) => State::Element {
+			Self::AttributeValue(kind, delim) => State::Element {
 				kind: kind,
-				state: ElementState::AttributeValue(delim, selector, false),
+				state: ElementState::AttributeValue(delim, false),
 			},
 			Self::Text => State::Content(ContentState::Initial),
 		}
@@ -610,7 +610,7 @@ impl Lexer {
 		}
 	}
 
-	fn read_validated<B: ByteSelect>(
+	fn read_validated<B: ByteSelect + ?Sized>(
 		&mut self,
 		r: &mut &[u8],
 		selector: &B,
@@ -848,7 +848,7 @@ impl Lexer {
 						))
 					}
 					byte => {
-						if CLASS_XML_NAMESTART_BYTE.select(byte) {
+						if maybe_name(byte) {
 							// add the first character to the scratchpad, because read_single does not do that
 							self.prep_scratchpad();
 							self.scratchpad.push(byte);
@@ -933,7 +933,7 @@ impl Lexer {
 			Some(st) => Ok(st),
 			// no special char -> check if it is possibly valid text and proceed accordingly
 			None => {
-				if CLASS_XML_MAY_NONCHAR_BYTE.select(b) {
+				if is_nonchar_byte(b) {
 					// non-Char, error
 					Err(Error::NotWellFormed(WFError::InvalidChar(
 						ERRCTX_TEXT,
@@ -1000,7 +1000,7 @@ impl Lexer {
 			self.scratchpad
 				.extend_from_slice(&TOK_XML_CDATA_END[..nend]);
 			if in_cdata {
-				if CLASS_XML_MAY_NONCHAR_BYTE.select(b) {
+				if is_nonchar_byte(b) {
 					// that’s a sneaky one!
 					Err(Error::NotWellFormed(WFError::InvalidChar(
 						ERRCTX_CDATA_SECTION,
@@ -1066,7 +1066,7 @@ impl Lexer {
 									State::Content(ContentState::MaybeCDataEnd(true, 1)),
 									None,
 								))
-							} else if !CLASS_XML_MAY_NONCHAR_BYTE.select(b) {
+							} else if !is_nonchar_byte(b) {
 								// ^ but of course we still need to check for a valid char. Thanks afl.
 								// no need for prep as we pushed above already
 								self.scratchpad.push(b);
@@ -1087,51 +1087,47 @@ impl Lexer {
 
 			// read until next `<` or `&`, which are the only things which
 			// can break us out of this state.
-			ContentState::Initial => match self.read_validated(
-				r,
-				&CLASS_XML_TEXT_DELIMITED_BYTE,
-				self.opts.max_token_length,
-			)? {
-				Endbyte::Eof => Ok(ST(State::Eof, self.maybe_flush_scratchpad_as_text(0)?)),
-				Endbyte::Limit => Ok(ST(
-					State::Content(ContentState::Initial),
-					self.maybe_flush_scratchpad_as_text(0)?,
-				)),
-				Endbyte::Delimiter(b) => match self.lex_posttext_char(b)? {
-					Some(st) => Ok(st),
-					// not a "special" char but not text either -> error
-					None => Err(Error::NotWellFormed(WFError::InvalidChar(
-						ERRCTX_TEXT,
-						b as u32,
-						false,
-					))),
-				},
-			},
-			ContentState::CDataSection => match self.read_validated(
-				r,
-				&CLASS_XML_CDATA_CDATASECTION_DELIMITED_BYTE,
-				self.opts.max_token_length,
-			)? {
-				Endbyte::Eof => Err(Error::wfeof(ERRCTX_CDATA_SECTION)),
-				Endbyte::Limit => Ok(ST(
-					State::Content(ContentState::CDataSection),
-					self.maybe_flush_scratchpad_as_text(0)?,
-				)),
-				// -> transition into the "first delimiter found" state
-				Endbyte::Delimiter(b) => match b {
-					b']' => Ok(ST(
-						State::Content(ContentState::MaybeCDataEnd(true, 1)),
-						None,
+			ContentState::Initial => {
+				match self.read_validated(r, &maybe_text, self.opts.max_token_length)? {
+					Endbyte::Eof => Ok(ST(State::Eof, self.maybe_flush_scratchpad_as_text(0)?)),
+					Endbyte::Limit => Ok(ST(
+						State::Content(ContentState::Initial),
+						self.maybe_flush_scratchpad_as_text(0)?,
 					)),
-					b'\r' => Ok(ST(State::Content(ContentState::MaybeCRLF(true)), None)),
-					_ => Err(Error::NotWellFormed(WFError::InvalidChar(
-						ERRCTX_CDATA_SECTION,
-						b as u32,
-						false,
-					))),
-				},
-			},
-			ContentState::Whitespace => match self.skip_matching(r, &CLASS_XML_SPACE_BYTE) {
+					Endbyte::Delimiter(b) => match self.lex_posttext_char(b)? {
+						Some(st) => Ok(st),
+						// not a "special" char but not text either -> error
+						None => Err(Error::NotWellFormed(WFError::InvalidChar(
+							ERRCTX_TEXT,
+							b as u32,
+							false,
+						))),
+					},
+				}
+			}
+			ContentState::CDataSection => {
+				match self.read_validated(r, &maybe_cdata_content, self.opts.max_token_length)? {
+					Endbyte::Eof => Err(Error::wfeof(ERRCTX_CDATA_SECTION)),
+					Endbyte::Limit => Ok(ST(
+						State::Content(ContentState::CDataSection),
+						self.maybe_flush_scratchpad_as_text(0)?,
+					)),
+					// -> transition into the "first delimiter found" state
+					Endbyte::Delimiter(b) => match b {
+						b']' => Ok(ST(
+							State::Content(ContentState::MaybeCDataEnd(true, 1)),
+							None,
+						)),
+						b'\r' => Ok(ST(State::Content(ContentState::MaybeCRLF(true)), None)),
+						_ => Err(Error::NotWellFormed(WFError::InvalidChar(
+							ERRCTX_CDATA_SECTION,
+							b as u32,
+							false,
+						))),
+					},
+				}
+			}
+			ContentState::Whitespace => match self.skip_matching(r, &is_space) {
 				(_, Ok(Endbyte::Eof)) | (_, Ok(Endbyte::Limit)) => Ok(ST(State::Eof, None)),
 				(_, Ok(Endbyte::Delimiter(b))) => match b {
 					b'<' => Ok(ST(
@@ -1152,16 +1148,8 @@ impl Lexer {
 	fn lex_element_postblank(&mut self, kind: ElementKind, b: u8) -> Result<ElementState> {
 		match b {
 			b' ' | b'\t' | b'\r' | b'\n' => Ok(ElementState::Blank),
-			b'"' => Ok(ElementState::AttributeValue(
-				b'"',
-				&CLASS_XML_CDATA_ATT_QUOT_DELIMITED_BYTE,
-				false,
-			)),
-			b'\'' => Ok(ElementState::AttributeValue(
-				b'\'',
-				&CLASS_XML_CDATA_ATT_APOS_DELIMITED_BYTE,
-				false,
-			)),
+			b'"' => Ok(ElementState::AttributeValue(b'"', false)),
+			b'\'' => Ok(ElementState::AttributeValue(b'\'', false)),
 			b'=' => Ok(ElementState::Eq),
 			b'>' => match kind {
 				ElementKind::Footer | ElementKind::Header => Ok(ElementState::Close),
@@ -1192,7 +1180,7 @@ impl Lexer {
 					None,
 				))),
 			},
-			b if CLASS_XML_NAMESTART_BYTE.select(b) => {
+			b if maybe_name(b) => {
 				// write the char to scratchpad because it’ll be needed.
 				self.prep_scratchpad();
 				self.scratchpad.push(b);
@@ -1209,13 +1197,7 @@ impl Lexer {
 		}
 	}
 
-	fn lex_attval_next(
-		&mut self,
-		delim: u8,
-		selector: &'static [ByteRange],
-		b: u8,
-		element_kind: ElementKind,
-	) -> Result<ST> {
+	fn lex_attval_next(&mut self, delim: u8, b: u8, element_kind: ElementKind) -> Result<ST> {
 		match b {
 			b'<' => Err(Error::NotWellFormed(WFError::UnexpectedChar(
 				ERRCTX_ATTVAL,
@@ -1229,7 +1211,7 @@ impl Lexer {
 				Ok(ST(
 					State::Reference {
 						ctx: ERRCTX_ATTVAL,
-						ret: RefReturnState::AttributeValue(element_kind, delim, selector),
+						ret: RefReturnState::AttributeValue(element_kind, delim),
 						kind: RefKind::Entity,
 					},
 					None,
@@ -1241,7 +1223,7 @@ impl Lexer {
 				Ok(ST(
 					State::Element {
 						kind: element_kind,
-						state: ElementState::AttributeValue(delim, selector, false),
+						state: ElementState::AttributeValue(delim, false),
 					},
 					None,
 				))
@@ -1249,7 +1231,7 @@ impl Lexer {
 			b'\r' => Ok(ST(
 				State::Element {
 					kind: element_kind,
-					state: ElementState::AttributeValue(delim, selector, true),
+					state: ElementState::AttributeValue(delim, true),
 				},
 				None,
 			)),
@@ -1275,64 +1257,33 @@ impl Lexer {
 	fn lex_element(&mut self, kind: ElementKind, state: ElementState, r: &mut &[u8]) -> Result<ST> {
 		match state {
 			ElementState::Start | ElementState::Name => {
-				if self.scratchpad.len() == 0 {
-					// we are reading the first char; the first one is special because it must match CLASS_XML_NAMESTART, and not just CLASS_XML_NAME
-					let b = handle_eof(self.read_single(r)?, ERRCTX_NAME)?;
-					if !CLASS_XML_NAMESTART_BYTE.select(b) {
-						Err(Error::NotWellFormed(WFError::UnexpectedByte(
-							ERRCTX_NAME,
-							b,
-							None,
-						)))
-					} else {
-						self.prep_scratchpad();
-						self.scratchpad.push(b);
-						// continue in the same state; the branch below will be taken next and read_validated will take care of it if we’re done already
+				match self.read_validated(r, &maybe_name, self.opts.max_token_length)? {
+					Endbyte::Eof => Err(Error::wfeof(ERRCTX_NAME)),
+					Endbyte::Limit => Err(Self::token_length_error()),
+					Endbyte::Delimiter(ch) => {
+						let next_state = self.lex_element_postblank(kind, ch)?;
+						let name = add_context(self.flush_scratchpad_as_name(), ERRCTX_NAMESTART)?;
+						let metrics = self.metrics(1);
 						Ok(ST(
 							State::Element {
 								kind: kind,
-								state: state,
+								state: next_state,
 							},
-							None,
+							Some(if state == ElementState::Name {
+								Token::Name(metrics, name)
+							} else {
+								match kind {
+									ElementKind::Header => Token::ElementHeadStart(metrics, name),
+									ElementKind::Footer => Token::ElementFootStart(metrics, name),
+									ElementKind::XMLDecl => panic!("invalid state"),
+								}
+							}),
 						))
 					}
-				} else {
-					match self.read_validated(
-						r,
-						&CLASS_XML_NAME_BYTE,
-						self.opts.max_token_length,
-					)? {
-						Endbyte::Eof => Err(Error::wfeof(ERRCTX_NAME)),
-						Endbyte::Limit => Err(Self::token_length_error()),
-						Endbyte::Delimiter(ch) => {
-							let next_state = self.lex_element_postblank(kind, ch)?;
-							let name = self.flush_scratchpad_as_name()?;
-							let metrics = self.metrics(1);
-							Ok(ST(
-								State::Element {
-									kind: kind,
-									state: next_state,
-								},
-								Some(if state == ElementState::Name {
-									Token::Name(metrics, name)
-								} else {
-									match kind {
-										ElementKind::Header => {
-											Token::ElementHeadStart(metrics, name)
-										}
-										ElementKind::Footer => {
-											Token::ElementFootStart(metrics, name)
-										}
-										ElementKind::XMLDecl => panic!("invalid state"),
-									}
-								}),
-							))
-						}
-					}
 				}
 			}
 			ElementState::SpaceRequired | ElementState::Blank => {
-				match self.skip_matching(r, &CLASS_XML_SPACE_BYTE) {
+				match self.skip_matching(r, &is_space) {
 					(_, Ok(Endbyte::Eof)) | (_, Ok(Endbyte::Limit)) => {
 						Err(Error::wfeof(ERRCTX_ELEMENT))
 					}
@@ -1373,17 +1324,20 @@ impl Lexer {
 				}
 			}
 			// XML 1.0 §2.3 [10] AttValue
-			ElementState::AttributeValue(delim, selector, false) => {
+			ElementState::AttributeValue(delim, false) => {
+				let selector = if delim == b'\'' {
+					&maybe_attval_apos as &dyn Fn(_) -> _
+				} else {
+					&maybe_attval_quot as &dyn Fn(_) -> _
+				};
 				match self.read_validated(r, &selector, self.opts.max_token_length)? {
 					Endbyte::Eof => Err(Error::wfeof(ERRCTX_ATTVAL)),
 					Endbyte::Limit => Err(Self::token_length_error()),
-					Endbyte::Delimiter(utf8ch) => {
-						self.lex_attval_next(delim, selector, utf8ch, kind)
-					}
+					Endbyte::Delimiter(utf8ch) => self.lex_attval_next(delim, utf8ch, kind),
 				}
 			}
 			// CRLF normalization for attributes; cannot reuse the element mechanism here because we have to carry around the delimiter and stuff
-			ElementState::AttributeValue(delim, selector, true) => {
+			ElementState::AttributeValue(delim, true) => {
 				let b = handle_eof(self.read_single(r)?, ERRCTX_ATTVAL)?;
 				if b == b'\r' {
 					// push the space, continue with CRLF
@@ -1392,13 +1346,13 @@ impl Lexer {
 					Ok(ST(
 						State::Element {
 							kind: kind,
-							state: ElementState::AttributeValue(delim, selector, true),
+							state: ElementState::AttributeValue(delim, true),
 						},
 						None,
 					))
 				} else {
 					// not another CR, so we can move on to the default handling
-					self.lex_attval_next(delim, selector, b, kind)
+					self.lex_attval_next(delim, b, kind)
 				}
 			}
 			ElementState::MaybeXMLDeclEnd => match self.read_single(r)? {
@@ -1458,14 +1412,12 @@ impl Lexer {
 		r: &mut &[u8],
 	) -> Result<ST> {
 		let result = match kind {
-			RefKind::Entity => {
-				self.read_validated(r, &CLASS_XML_NAME_BYTE, MAX_REFERENCE_LENGTH)?
-			}
+			RefKind::Entity => self.read_validated(r, &maybe_name, MAX_REFERENCE_LENGTH)?,
 			RefKind::Char(CharRefRadix::Decimal) => {
-				self.read_validated(r, &CLASS_XML_DECIMAL_DIGIT_BYTE, MAX_REFERENCE_LENGTH)?
+				self.read_validated(r, &is_decimal_digit, MAX_REFERENCE_LENGTH)?
 			}
 			RefKind::Char(CharRefRadix::Hexadecimal) => {
-				self.read_validated(r, &CLASS_XML_HEXADECIMAL_DIGIT_BYTE, MAX_REFERENCE_LENGTH)?
+				self.read_validated(r, &is_hexadecimal_digit, MAX_REFERENCE_LENGTH)?
 			}
 		};
 		let result = match result {
@@ -1631,10 +1583,45 @@ impl Lexer {
 	///
 	/// Returns `None` if a valid end of file is reached, a token if a valid
 	/// token is encountered or an error otherwise.
+	pub fn lex_buffer<T: bytes::Buf>(
+		&mut self,
+		r: &mut T,
+		at_eof: bool,
+	) -> CrateResult<Option<Token>> {
+		loop {
+			let mut chunk = r.chunk();
+			let prev_len = chunk.len();
+			// Only consider eof if the current chunk is truly the last one, which can be determined by checking that the chunk contains all remaining bytes.
+			self.has_eof = at_eof && prev_len == r.remaining();
+			let result = self.lex_bytes_raw(&mut chunk);
+			let new_len = chunk.len();
+			r.advance(prev_len - new_len);
+			match result {
+				Err(Error::EndOfBuffer) => {
+					if r.remaining() > 0 {
+						// more to read, but probably not in this chunk -> iterate
+						continue;
+					} else {
+						// nothing more to read, return wouldblock
+						return Ok(result?);
+					}
+				}
+				// any other result is to be returned immediately
+				other => return Ok(other?),
+			}
+		}
+	}
+
+	/// Alias of [`lex_buffer()`].
+	///
+	///    [`lex_buffer()`]: Self::lex_buffer
+	#[deprecated(
+		since = "0.7.0",
+		note = "use `lex_buffer` instead (`&[u8]` implements `Buf`)"
+	)]
 	#[inline]
 	pub fn lex_bytes(&mut self, r: &mut &[u8], at_eof: bool) -> CrateResult<Option<Token>> {
-		self.has_eof = at_eof;
-		Ok(self.lex_bytes_raw(r)?)
+		self.lex_buffer(r, at_eof)
 	}
 
 	/// Lex bytes from the reader until either an error occurs, a valid
@@ -1691,7 +1678,7 @@ impl Lexer {
 				Ok(b) => (b, b.len() == 0),
 			};
 			let orig_len = buf.len();
-			let result = self.lex_bytes(&mut buf, eof);
+			let result = self.lex_buffer(&mut buf, eof);
 			let new_len = buf.len();
 			assert!(new_len <= orig_len);
 			r.consume(orig_len - new_len);
@@ -1737,42 +1724,39 @@ pub trait Sink {
 #[cfg(test)]
 mod tests {
 	use super::*;
-	use crate::bufq::BufferQueue;
 	use crate::error::Error as CrateError;
 	use std::error;
 	use std::fmt;
 	use std::io;
 
 	/// Stream tokens to the sink until the end of stream is reached.
-	fn stream_to_sink<'r, 's, 'l, R: io::BufRead, S: Sink>(
-		l: &'l mut Lexer,
-		r: &'r mut R,
-		s: &'s mut S,
+	fn stream_to_sink<R: bytes::Buf, S: Sink>(
+		l: &mut Lexer,
+		r: &mut R,
+		s: &mut S,
+		at_eof: bool,
 	) -> CrateResult<()> {
 		loop {
-			match l.lex(r) {
-				Ok(Some(tok)) => s.token(tok),
-				Ok(None) => break,
-				Err(CrateError::IO(e)) if e.kind() == io::ErrorKind::WouldBlock => {
-					if let Ok(buf) = r.fill_buf() {
-						if buf.len() > 0 {
-							continue;
-						}
-					}
-					return Err(CrateError::IO(e));
-				}
-				Err(e) => return Err(e),
+			match l.lex_buffer(r, at_eof)? {
+				Some(tok) => s.token(tok),
+				None => break,
 			}
 		}
 		Ok(())
 	}
 
-	fn stream_to_sink_from_bytes<'r, 's, 'l, R: io::BufRead, S: Sink>(
-		l: &'l mut Lexer,
-		r: &'r mut R,
-		s: &'s mut S,
+	fn stream_to_sink_from_bytes<R: io::BufRead, S: Sink>(
+		l: &mut Lexer,
+		r: &mut R,
+		s: &mut S,
 	) -> CrateResult<()> {
-		stream_to_sink(l, r, s)
+		loop {
+			match l.lex(r)? {
+				Some(tok) => s.token(tok),
+				None => break,
+			}
+		}
+		Ok(())
 	}
 
 	struct VecSink {
@@ -1815,28 +1799,26 @@ mod tests {
 		}
 	}
 
-	fn lex(data: &[u8], token_limit: usize) -> (Vec<Token>, CrateResult<()>) {
-		let mut buff = io::BufReader::new(data);
+	fn lex(mut data: &[u8], token_limit: usize) -> (Vec<Token>, CrateResult<()>) {
 		let mut lexer = Lexer::new();
 		let mut sink = VecSink::new(token_limit);
-		let result = stream_to_sink(&mut lexer, &mut buff, &mut sink);
+		let result = stream_to_sink(&mut lexer, &mut data, &mut sink, true);
 		(sink.dest, result)
 	}
 
 	fn lex_chunked(data: &[&[u8]], token_limit: usize) -> (Vec<Token>, CrateResult<()>) {
-		let mut buff = BufferQueue::new();
 		let mut lexer = Lexer::new();
 		let mut sink = VecSink::new(token_limit);
 		for chunk in data.iter() {
-			buff.push(*chunk);
-			match stream_to_sink(&mut lexer, &mut buff, &mut sink) {
+			let mut chunk = *chunk;
+			match stream_to_sink(&mut lexer, &mut chunk, &mut sink, false) {
 				Ok(()) => panic!("unexpected end of tokens"),
 				Err(CrateError::IO(ioerr)) if ioerr.kind() == io::ErrorKind::WouldBlock => (),
 				Err(e) => return (sink.dest, Err(e)),
 			}
+			assert_eq!(chunk.len(), 0);
 		}
-		buff.push_eof();
-		let result = stream_to_sink(&mut lexer, &mut buff, &mut sink);
+		let result = stream_to_sink(&mut lexer, &mut &[][..], &mut sink, true);
 		(sink.dest, result)
 	}
 
@@ -1845,11 +1827,10 @@ mod tests {
 		r.err()
 	}
 
-	fn run_fuzz_test(data: &[u8], token_limit: usize) -> CrateResult<Vec<Token>> {
-		let mut buff = io::BufReader::new(data);
+	fn run_fuzz_test(mut data: &[u8], token_limit: usize) -> CrateResult<Vec<Token>> {
 		let mut lexer = Lexer::new();
 		let mut sink = VecSink::new(token_limit);
-		stream_to_sink(&mut lexer, &mut buff, &mut sink)?;
+		stream_to_sink(&mut lexer, &mut data, &mut sink, true)?;
 		Ok(sink.dest)
 	}
 
@@ -2567,19 +2548,19 @@ mod tests {
 	fn lexer_rejects_invalid_namestarts() {
 		let err = lex_err(b"<123/>", 128).unwrap();
 		match err {
-			CrateError::NotWellFormed(WFError::UnexpectedByte(..)) => (),
+			CrateError::NotWellFormed(WFError::UnexpectedChar(_, '1', None)) => (),
 			other => panic!("unexpected error: {:?}", other),
 		}
 
 		let err = lex_err(b"<'foo/>", 128).unwrap();
 		match err {
-			CrateError::NotWellFormed(WFError::UnexpectedByte(..)) => (),
+			CrateError::NotWellFormed(WFError::UnexpectedByte(_, b'\'', None)) => (),
 			other => panic!("unexpected error: {:?}", other),
 		}
 
 		let err = lex_err(b"<.bar/>", 128).unwrap();
 		match err {
-			CrateError::NotWellFormed(WFError::UnexpectedByte(..)) => (),
+			CrateError::NotWellFormed(WFError::UnexpectedChar(_, '.', None)) => (),
 			other => panic!("unexpected error: {:?}", other),
 		}
 	}
@@ -2677,16 +2658,16 @@ mod tests {
 	#[test]
 	fn lexer_rejects_non_xml_10_chars_verbatim_in_attrs() {
 		let err = lex_err(b"<a foo='\x00'/>", 128).unwrap();
-		assert!(matches!(
-			err,
-			CrateError::NotWellFormed(WFError::InvalidChar(_, _, false))
-		));
+		match err {
+			CrateError::NotWellFormed(WFError::InvalidChar(_, _, false)) => (),
+			other => panic!("unexpected error: {:?}", other),
+		}
 
 		let err = lex_err(b"<a foo='\x1f'/>", 128).unwrap();
-		assert!(matches!(
-			err,
-			CrateError::NotWellFormed(WFError::InvalidChar(_, _, false))
-		));
+		match err {
+			CrateError::NotWellFormed(WFError::InvalidChar(_, _, false)) => (),
+			other => panic!("unexpected error: {:?}", other),
+		}
 	}
 
 	#[test]
@@ -2736,22 +2717,8 @@ mod tests {
 	#[test]
 	fn lexer_recovers_from_wouldblock() {
 		let seq = &b"<?xml version='1.0'?>"[..];
-		let mut r = BufferQueue::new();
-		let mut lexer = Lexer::new();
-		let mut sink: Vec<Token> = Vec::new();
-		for chunk in seq.chunks(5) {
-			r.push(std::borrow::Cow::from(chunk));
-			loop {
-				match lexer.lex(&mut r) {
-					Err(CrateError::IO(ioerr)) if ioerr.kind() == io::ErrorKind::WouldBlock => {
-						break
-					}
-					Err(other) => panic!("unexpected error: {:?}", other),
-					Ok(None) => panic!("unexpected eof signal: {:?}", lexer),
-					Ok(Some(tok)) => sink.push(tok),
-				}
-			}
-		}
+		let (sink, result) = lex_chunked(&seq.chunks(5).collect::<Vec<_>>(), 128);
+		result.unwrap();
 
 		let mut iter = sink.iter();
 		assert_eq!(
@@ -2785,22 +2752,8 @@ mod tests {
 	#[test]
 	fn lexer_recovers_from_wouldblock_within_long_whitespace_with_correct_counting() {
 		let seq = &b"<?xml   version  =  '1.0'  ?>"[..];
-		let mut r = BufferQueue::new();
-		let mut lexer = Lexer::new();
-		let mut sink: Vec<Token> = Vec::new();
-		for chunk in seq.chunks(5) {
-			r.push(std::borrow::Cow::from(chunk));
-			loop {
-				match lexer.lex(&mut r) {
-					Err(CrateError::IO(ioerr)) if ioerr.kind() == io::ErrorKind::WouldBlock => {
-						break
-					}
-					Err(other) => panic!("unexpected error: {:?}", other),
-					Ok(None) => panic!("unexpected eof signal: {:?}", lexer),
-					Ok(Some(tok)) => sink.push(tok),
-				}
-			}
-		}
+		let (sink, result) = lex_chunked(&seq.chunks(5).collect::<Vec<_>>(), 128);
+		result.unwrap();
 
 		let mut iter = sink.iter();
 		assert_eq!(
@@ -3016,24 +2969,44 @@ mod tests {
 		));
 	}
 
+	#[test]
+	fn lexer_rejects_numeric_start_of_name_in_opening_tag() {
+		// found via fuzzing by moparisthebest
+		let err = lex_err(b"<4foo>", 128).unwrap();
+		match err {
+			CrateError::NotWellFormed(WFError::UnexpectedChar(_, '4', None)) => (),
+			other => panic!("unexpected error: {:?}", other),
+		}
+	}
+
 	#[test]
 	fn lexer_rejects_numeric_start_of_name_in_closing_tag() {
 		// found via fuzzing by moparisthebest
 		let err = lex_err(b"</4foo>", 128).unwrap();
-		assert!(matches!(
-			err,
-			CrateError::NotWellFormed(WFError::UnexpectedByte(_, b'4', None))
-		));
+		match err {
+			CrateError::NotWellFormed(WFError::UnexpectedChar(_, '4', None)) => (),
+			other => panic!("unexpected error: {:?}", other),
+		}
+	}
+
+	#[test]
+	fn lexer_rejects_zero_length_name_in_opening_tag() {
+		// found via fuzzing by moparisthebest
+		let err = lex_err(b"< >", 128).unwrap();
+		match err {
+			CrateError::NotWellFormed(WFError::UnexpectedByte(_, b' ', None)) => (),
+			other => panic!("unexpected error: {:?}", other),
+		}
 	}
 
 	#[test]
 	fn lexer_rejects_zero_length_name_in_closing_tag() {
 		// found via fuzzing by moparisthebest
 		let err = lex_err(b"</ >", 128).unwrap();
-		assert!(matches!(
-			err,
-			CrateError::NotWellFormed(WFError::UnexpectedByte(_, b' ', None))
-		));
+		match err {
+			CrateError::NotWellFormed(WFError::InvalidSyntax(_)) => (),
+			other => panic!("unexpected error: {:?}", other),
+		}
 	}
 
 	#[test]
@@ -3315,18 +3288,17 @@ mod tests {
 
 	#[test]
 	fn lexer_emits_close_tag_token_even_at_end_of_buffer() {
-		let mut buf = BufferQueue::new();
-		buf.push(&b"</foo>"[..]);
+		let mut buf = &b"</foo>"[..];
 		let mut lexer = Lexer::new();
-		match lexer.lex(&mut buf) {
+		match lexer.lex_buffer(&mut buf, false) {
 			Ok(Some(Token::ElementFootStart(_, _))) => (),
 			other => panic!("unexpected result: {:?}", other),
 		};
-		match lexer.lex(&mut buf) {
+		match lexer.lex_buffer(&mut buf, false) {
 			Ok(Some(Token::ElementHFEnd(_))) => (),
 			other => panic!("unexpected result: {:?}", other),
 		};
-		match lexer.lex(&mut buf) {
+		match lexer.lex_buffer(&mut buf, false) {
 			Err(CrateError::IO(ioerr)) if ioerr.kind() == io::ErrorKind::WouldBlock => (),
 			other => panic!("unexpected result: {:?}", other),
 		};
